/*
=====================================================================
	5D6.CPP 
    Results for 5d6, drop 2 (average score: 13.43; point buy: 38.4)
	Copyright (c) 2005 Daniel R. Collins. All rights reserved.
	See the bottom of this file for any licensing information.
=====================================================================
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "dice.h"


/*
=====================================================================
  D&D functions
=====================================================================
*/

// Ability modifier for one score

int abilityMod (int ability) {
  return ability / 2 - 5;
}


// Point-buy value for ability

int pointValue (int ability) {
  assert(ability <= 18);
  switch (ability) {
    default:  return 0;
    case  9:  return 1;
    case 10:  return 2;
    case 11:  return 3;
    case 12:  return 4;
    case 13:  return 5;
    case 14:  return 6;
    case 15:  return 8;
    case 16:  return 10;
    case 17:  return 13;
    case 18:  return 16;
  }
}


/*
=====================================================================
  Main functions
=====================================================================
*/

// Compare integers

int compareInts (const void *x, const void *y) {
  int a = *(int*)x;
  int b = *(int*)y;
  return (a<b ? -1 : (a==b ? 0 : 1));
}


// One roll

int oneRoll (void) {
  int dice[5];
  for (int i = 0; i < 5; i++) {
    dice[i] = Dice::roll(6);
  }
  qsort(dice, 5, sizeof(int), &compareInts);
  return dice[2] + dice[3] + dice[4];
}


// One character (with hopeless check)
// Returns point-buy value

int oneCharacter (void) {
  int score[6];
  bool accept = false;
  while (!accept) {

    // Roll scores  
    int highest = 0;
    int totalMod = 0;
    for (int i = 0; i < 6; i++) {
      score[i] = oneRoll();
      totalMod += score[i];
      if (score[i] > highest)
        highest = score[i];
    }

    // Accept if not helpless
    if (!(totalMod <= 0 || highest <= 13))
      accept = true;
  }

  // Sum point-buy scores
  int pointTotal = 0;
  for (int i = 0; i < 6; i++) {
    pointTotal += pointValue(score[i]);
  }
  return pointTotal;
}


// Many rolls: returns average

double manyRolls (void) {
  const int TRIALS = 1000000;
  int sum = 0;
  for (int i = 0; i < TRIALS; i++) {
    sum += oneCharacter();
  }
  return (double) sum / TRIALS;
}


// Main function

int main (int argc, char *argv[])
	{
  Dice::init();
  printf("%f\n", manyRolls());
	return 0;
	}

