/*
==========================================================================
	DICE.CPP (version 1.06)
		Standard dice-rolling library.
	Copyright (c) 2001-2004 by Daniel R. Collins. All rights reserved.
  You may freely copy this document, if copyright notice is maintained.
==========================================================================
*/

#include "dice.h"
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>


/*
---------------------------------------
  Static functions
---------------------------------------
*/

void Dice::init (void) {
  srand(time(NULL));
}


int Dice::roll (int type) {
	return 1 + rand()%type;
}


int Dice::roll (int num, int type) {
	int sum = num;
	for (int i = 0; i < num; i++)
		sum += rand()%type;
	return sum;
}


/*
---------------------------------------
  Constructors
---------------------------------------
*/

Dice::Dice (void) {
  set(0, 0, 0);
}


Dice::Dice (int _num, int _type, int _mod) {
  set(_num, _type, _mod);
}


Dice::Dice (const char *s) {
  read(s);
}


/*
---------------------------------------
  Member functions
---------------------------------------
*/

// Set the dice.

void Dice::set (int _num, int _type, int _mod) {
  num = _num; 
  type = _type; 
  mod = _mod; 
  reserved = 0;
}


// Read RPG dice shorthand string in "nDt+m" format.

void Dice::read (const char *s) {
  assert(s);

  // Init members
  num = type = mod = 0;

	// Search for 'd' in string
	char *p = strchr(s, 'd');
	if (!p) p = strchr(s, 'D');
  
  if (p) {
    // Parse number
    num = (p == s ? 1 : atoi(s));

    // Parse die type
    p++;
    type = (*p == '%' ? 100 : atoi(p));

    // Parse modifier (find '+' or '-')
    while (*p) {
      if (*p == '+' || *p == '-') {
        mod = atoi(p);
        break;
      }
      else if (*p != ' ' && !isdigit(*p)) {
        break;
      }
      p++;
    }
  }
  else {
    // No 'd' found, it's just a constant number
    mod = atoi(s);
  }
}


// Write RPG dice shorthand string in "nDt+m" format.

char *Dice::write (char *s) {
  assert(s);

  if (num > 0 && mod != 0) {
    sprintf(s, "%dd%d%+d", num, type, mod);
  }
  else if (num > 0) {
    sprintf(s, "%dd%d", num, type);
  }
  else {
    sprintf(s, "%d", mod);
  }

  return s;
}


// Roll the dice specified.

int Dice::roll (void) {
  return roll(num, type) + mod;
}


// Roll dice with minimum of 1 per die.

int Dice::rollMin1 (void) {
  int sum = 0;
  int dieMod = mod/num;
  for (int i = 0; i < num; i++) {
    int die = roll(type) + dieMod;
    sum += (die > 1 ? die : 1);
  }
  return sum;
}

